@@ -18,14 +18,13 @@ from storage import Storage, List
 from streamer import streamer, stream_file_or_304_or_206, DEFAULT_CHUNK_SIZE
 from xmlrpc import handler
 from contenttype import contenttype
-from html import xmlescape, TABLE, TR, PRE
-from http import HTTP
+from html import xmlescape, TABLE, TR, PRE, URL
+from http import HTTP, redirect
 from fileutils import up
 from serializers import json, custom_json
 import settings
 from utils import web2py_uuid
 from settings import global_settings
-
 import hashlib
 import portalocker
 import cPickle
@@ -38,12 +37,23 @@ import sys
 import traceback
 import threading
 
+try:
+    from gluon.contrib.minify import minify
+    have_minify = True
+except ImportError:
+    have_minify = False
+
 regex_session_id = re.compile('^([\w\-]+/)?[\w\-\.]+$')
 
 __all__ = ['Request', 'Response', 'Session']
 
 current = threading.local()  # thread-local storage for request-scope globals
 
+css_template = '<link href="%s" rel="stylesheet" type="text/css" />'
+js_template = '<script src="%s" type="text/javascript"></script>'
+css_inline = '<style type="text/css">\n%s\n</style>'
+js_inline = '<script type="text/javascript">\n%s\n</script>'
+
 class Request(Storage):
 
     """
@@ -93,9 +103,22 @@ class Request(Storage):
     def user_agent(self):
         from gluon.contrib import user_agent_parser
         session = current.session
-        session._user_agent = session._user_agent or \
+        user_agent = session._user_agent = session._user_agent or \
             user_agent_parser.detect(self.env.http_user_agent)
-        return session._user_agent
+        user_agent = Storage(user_agent)
+        for key,value in user_agent.items():
+            if isinstance(value,dict): user_agent[key] = Storage(value)
+        return user_agent
+        
+    def requires_https(self):
+        """
+        If request comes in over HTTP, redirect it to HTTPS
+        and secure the session.
+        """
+        if not global_settings.cronjob and not self.is_https:
+            redirect(URL(scheme='https', args=self.args, vars=self.vars))
+        
+        current.session.secure()
 
     def restful(self):
         def wrapper(action,self=self):
@@ -193,16 +216,44 @@ class Response(Storage):
         self.write(s,escape=False)
 
     def include_files(self):
+
+
+        """
+        Caching method for writing out files.
+        By default, caches in ram for 5 minutes. To change,
+        response.cache_includes = (cache_method, time_expire).
+        Example: (cache.disk, 60) # caches to disk for 1 minute.
+        """
+        from gluon import URL
+
+        files = []
+        for item in self.files:
+            if not item in files: files.append(item)
+        if have_minify and (self.optimize_css or self.optimize_js):
+            # cache for 5 minutes by default
+            cache = self.cache_includes or (current.cache.ram, 60*5)
+            def call_minify():
+                return minify.minify(files,URL('static','temp'),
+                                     current.request.folder,
+                                     self.optimize_css,self.optimize_js)
+            if cache:
+                cache_model, time_expire = cache
+                files = cache_model('response.files.minified',call_minify,
+                                    time_expire)
+            else:
+                files = call_minify()
         s = ''
-        for k,f in enumerate(self.files or []):
-            if not f in self.files[:k]:
-                filename = f.lower().split('?')[0]
-                if filename.endswith('.css'):
-                    s += '<link href="%s" rel="stylesheet" type="text/css" />' % f
-                elif filename.endswith('.js'):
-                    s += '<script src="%s" type="text/javascript"></script>' % f
-        self.write(s,escape=False)
-    
+        for item in files:
+            if isinstance(item,str):
+                f = item.lower()
+                if f.endswith('.css'):  s += css_template % item
+                elif f.endswith('.js'): s += js_template % item
+            elif isinstance(item,(list,tuple)):
+                f = item[0]
+                if f=='css:inline':     s += css_inline % item[1]
+                elif f=='js:inline':    s += js_inline % item[1]
+        self.write(s, escape=False)
+
     def stream(
         self,
         stream,
@@ -216,7 +267,8 @@ class Response(Storage):
 
         the file content will be streamed at 100 bytes at the time
         """
-
+        if not request:
+            request = current.request
         if isinstance(stream, (str, unicode)):
             stream_file_or_304_or_206(stream,
                                       chunk_size=chunk_size,
@@ -238,6 +290,14 @@ class Response(Storage):
                     os.path.getsize(filename)
             except OSError:
                 pass
+
+        # Internet Explorer < 9.0 will not allow downloads over SSL unless caching is enabled
+        if request.is_https and isinstance(request.env.http_user_agent,str) and \
+                not re.search(r'Opera', request.env.http_user_agent) and \
+                re.search(r'MSIE [5-8][^0-9]', request.env.http_user_agent):
+            self.headers['Pragma'] = 'cache'
+            self.headers['Cache-Control'] = 'private'
+
         if request and request.env.web2py_use_wsgi_file_wrapper:
             wrapped = request.env.wsgi_file_wrapper(stream, chunk_size)
         else:
@@ -305,10 +365,13 @@ class Response(Storage):
         admin = URL("admin","default","design",
                     args=current.request.application)
         from gluon.dal import thread
-        dbstats = [TABLE(*[TR(PRE(row[0]),'%.2fms' % (row[1]*1000)) \
-                               for row in i.db._timings]) \
-                       for i in thread.instances]        
-        u = web2py_uuid() 
+        if hasattr(thread,'instances'):
+            dbstats = [TABLE(*[TR(PRE(row[0]),'%.2fms' % (row[1]*1000)) \
+                                   for row in i.db._timings]) \
+                           for i in thread.instances]
+        else:
+            dbstats = [] # if no db or on GAE
+        u = web2py_uuid()
         return DIV(
             BUTTON('design',_onclick="document.location='%s'" % admin),
             BUTTON('request',_onclick="jQuery('#request-%s').slideToggle()"%u),
@@ -355,7 +418,7 @@ class Session(Storage):
             if global_settings.db_sessions is True or masterapp in global_settings.db_sessions:
                 return
             response.session_new = False
-            client = request.client.replace(':', '.')
+            client = request.client and request.client.replace(':', '.')
             if response.session_id_name in request.cookies:
                 response.session_id = \
                     request.cookies[response.session_id_name].value
@@ -547,3 +610,4 @@ class Session(Storage):
                 pass
 
 
+
