@@ -29,9 +29,6 @@ from storage import Storage
 from highlight import highlight
 from utils import web2py_uuid, hmac_hash
 
-import hmac
-import hashlib
-
 regex_crlf = re.compile('\r|\n')
 
 join = ''.join
@@ -230,7 +227,10 @@ def URL(
         controller = c
     if f:
         if not isinstance(f, str):
-            function = f.__name__
+            if hasattr(f,'__name__'):
+                function = f.__name__
+            else:
+                raise SyntaxError, 'when calling URL, function or function name required'
         elif '.' in f:
             function, extension = f.split('.', 1)
         else:
@@ -347,7 +347,7 @@ def verifyURL(request, hmac_key=None, hash_vars=True, salt=None, user_signature=
     # check if user_signature requires
     if user_signature:
         from globals import current
-        if not current.session:
+        if not current.session or not current.session.auth:
             return False
         hmac_key = current.session.auth.hmac_key
     if not hmac_key:
@@ -1310,8 +1310,8 @@ class A(DIV):
                     (self['callback'],self['target'] or '',d)
             self['_href'] = self['_href'] or '#null'
         elif self['cid']:
-            self['_onclick']='web2py_component("%s","%s");return false;' % \
-                (self['_href'],self['cid'])
+            self['_onclick']='web2py_component("%s","%s");%sreturn false;' % \
+                (self['_href'],self['cid'],d)
         return DIV.xml(self)
 
 
@@ -1904,9 +1904,10 @@ class FORM(DIV):
             elif callable(onsuccess):
                 onsuccess(self)
             if next:
-                if self.vars.id:
-                    next = next.replace('[id]',str(self.vars.id))
-                    next = next % self.vars
+                if self.vars:
+                    for key,value in self.vars.items():
+                        next = next.replace('[%s]' % key,
+                                            urllib.quote(str(value)))
                     if not next.startswith('/'):
                         next = URL(next)
                 redirect(next)
@@ -2004,7 +2005,7 @@ class BEAUTIFY(DIV):
                         value = c[key]
                         if type(value) == types.LambdaType:
                             continue
-                        rows.append(TR(TD(filtered_key, _style='font-weight:bold;'),
+                        rows.append(TR(TD(filtered_key, _style='font-weight:bold;vertical-align:top'),
                                        TD(':',_valign='top'),
                                        TD(BEAUTIFY(value, **attributes))))
                     components.append(TABLE(*rows, **attributes))
@@ -2053,6 +2054,8 @@ class MENU(DIV):
             self['li_class'] = 'web2py-menu-expand'
         if not 'li_active' in self.attributes:
             self['li_active'] = 'web2py-menu-active'
+        if not 'mobile' in self.attributes:
+            self['mobile'] = False
 
     def serialize(self, data, level=0):
         if level == 0:
@@ -2081,8 +2084,22 @@ class MENU(DIV):
             ul.append(li)
         return ul
 
+    def serialize_mobile(self, data, select=None, prefix=''):
+        if not select:
+            select = SELECT()
+        for item in data:
+            if item[2]:
+                select.append(OPTION(CAT(prefix, item[0]), _value=item[2], _selected=item[1]))
+            if len(item)>3 and len(item[3]):
+                self.serialize_mobile(item[3], select, prefix = CAT(prefix, item[0], '/'))
+        select['_onchange'] = 'window.location=this.value'
+        return select
+
     def xml(self):
-        return self.serialize(self.data, 0).xml()
+        if self['mobile']:
+            return self.serialize_mobile(self.data, 0).xml()
+        else:
+            return self.serialize(self.data, 0).xml()
 
 
 def embed64(
@@ -2184,10 +2201,12 @@ class web2pyHTMLParser(HTMLParser):
         else:
             self.last = tag.tag[:-1]
     def handle_data(self,data):
-        try:
-            self.parent.append(data.encode('utf8','xmlcharref'))
-        except:
-            self.parent.append(data.decode('latin1').encode('utf8','xmlcharref'))
+        if not isinstance(data,unicode):
+            try:
+                data = data.decode('utf8')
+            except:
+                data = data.decode('latin1')
+        self.parent.append(data.encode('utf8','xmlcharref'))
     def handle_charref(self,name):
         if name[1].lower()=='x':
             self.parent.append(unichr(int(name[2:], 16)).encode('utf8'))
@@ -2284,3 +2303,4 @@ if __name__ == '__main__':
 
 
 
+
