@@ -93,7 +93,8 @@ def write_dict(filename, contents):
     try:
         fp = open(filename, 'w')
     except IOError:
-        logging.error('Unable to write to file %s' % filename)
+        if not is_gae:
+            logging.warning('Unable to write to file %s' % filename)
         return
     portalocker.lock(fp, portalocker.LOCK_EX)
     fp.write('# coding: utf8\n{\n')
@@ -250,9 +251,11 @@ class translator(object):
         self.t = {}  # ## no language by default
         return languages
 
-    def __call__(self, message, symbols={}, language=None):
+    def __call__(self, message, symbols={}, language=None, lazy=None):
+        if lazy is None:
+            lazy = self.lazy
         if not language:
-            if self.lazy:
+            if lazy:
                 return lazyT(message, symbols, self)
             else:
                 return self.translate(message, symbols)
@@ -262,7 +265,7 @@ class translator(object):
             except KeyError:
                 otherT = self.otherTs[language] = translator(self.request)
                 otherT.force(language)
-            return otherT(message,symbols)
+            return otherT(message, symbols, lazy=lazy)
 
     def translate(self, message, symbols):
         """
@@ -351,3 +354,4 @@ if __name__ == '__main__':
 
 
 
+
