@@ -35,7 +35,7 @@ class Storage(dict):
 
     """
     A Storage object is like a dictionary except `obj.foo` can be used
-    in addition to `obj['foo']`.
+    in addition to `obj['foo']`, and setting obj.foo = None deletes item foo.
 
         >>> o = Storage(a=1)
         >>> print o.a
@@ -55,10 +55,7 @@ class Storage(dict):
     """
 
     def __getattr__(self, key):
-        if key in self:
-            return self[key]
-        else:
-            return None
+        return dict.get(self, key, None)
 
     def __setattr__(self, key, value):
         if value is None:
@@ -73,6 +70,9 @@ class Storage(dict):
         else:
             raise AttributeError, "missing key=%s" % key
 
+    def __getitem__(self, key):
+        return dict.get(self, key, None)
+
     def __repr__(self):
         return '<Storage ' + dict.__repr__(self) + '>'
 
@@ -224,3 +224,4 @@ if __name__ == '__main__':
 
 
 
+
